//
// ComponentDesign.h
// 

#ifndef _ComponentDesign_H
#define _ComponentDesign_H

#include <InterfaceKit.h>
#include <List.h>
#include <Menu.h>
#include <PopUpMenu.h>
#include <Alert.h>
#include "StringList.h"
#include <string.h>
#include <Alert.h>
#include <String.h>

enum {
	DTM_COMPONENT_MOVE  = 0x999
, DTM_COMPONENT_RESIZE 
, DTM_COMPONENT_CONTEXTMENU
};

#define T_PROP_VALUE_LENGTH 1024
#define T_PROP_NAME_LENGTH 	32
#define T_COMPONENT_CLASSNAME_LENGTH 32
#define RESIZE_OFFSET 5
#define MIN_COMPONENT_SIZE 8

const rgb_color clSelectionRect = {0,200,200,255};
const rgb_color clLockedSelectionRect = {0,150,150,255};

class TComponent;
class TComponentOwner;
class TPropertyEditor;
class TFormDesigner;

//
// These control how a property is edited in the properties window.
//
enum TPropEditType {
	petString, 		// std. string property
	petModalEdit, 	// Displays a modal dialog
	petList,				// Displays a menu field with multiple entries; User can pick multiple items.
	petEnum,				// Displays a choice box with component-supplied values; User can pick one item.
	petComponent		// Displays a choice box with the names of all components on the 
									// form listed.
};
#define T_NUM_PROP_TYPES 5

//
// Component mode. Tells the component whether it's design time or runtime. This is useful
// If a control doesn't want to do something other than at runtime in response to a property being
// set.
//
enum TComponentMode {
	cmDesign, cmRuntime
};

//
// Component type. Determines if the component is shown on the form at runtime or not
//
enum TComponentType {
	ctVisual, ctNonVisual
};

//
// Property storage. When a property is set, the value is stored in this object.
//
class TPropertyStore: public BMessage {
public:
	TPropertyStore(TComponent *component);
	virtual ~TPropertyStore();
	
	status_t	SetBool(const char *name, bool b);
	status_t	SetInt8(const char *name, int8 i);
	status_t	SetInt16(const char *name, int16 i);
	status_t	SetInt32(const char *name, int32 i);
	status_t	SetInt64(const char *name, int64 i);
	status_t	SetFloat(const char *name, float f);
	status_t	SetDouble(const char *name, double d);
	status_t	SetString(const char *name, const char *string);
	status_t	SetPoint(const char *name, BPoint p);
	status_t	SetRect(const char *name, BRect r);
	status_t	SetMessage(const char *name, const BMessage *message);
	status_t	SetFlat(const char *name, const BFlattenable *obj);
	
	void SetComponent(TComponent *component) { m_Component = component; }
	TComponent *Component(void) const { return m_Component; }
private:
	TComponent *m_Component;
};


//
// A Property. A list of these is stored in the TComponent
//
class TProperty {
public:
	TProperty(
		TComponent *cOwner
	, const char *sName
	,	const char *propValue = ""
	,	TPropertyEditor *editor = NULL
	, bool bGenerated = true
	)
	: m_Owner(cOwner)
	, m_Editor(editor)
	, m_Generated(bGenerated)
	{ 
		if(sName) strncpy(m_Name, sName, sizeof(m_Name));
		if(propValue) 
		{
			strncpy(m_Value, propValue, sizeof(m_Value)); 
			strncpy(m_DefValue, propValue, sizeof(m_Value));
		}
		else
		{
			memset(m_Value, 0, sizeof(m_Value));
			memset(m_DefValue, 0, sizeof(m_DefValue));
		}
		
		m_Editor = editor;
		
		m_Quoted = false;
		memset(m_Get, 0, sizeof(m_Get));
		memset(m_Set, 0, sizeof(m_Set));
	}
	
	TProperty(TProperty &other)
	: m_Owner((TComponent *)other.Owner())
	, m_Editor(other.Editor())
	{ 
		strncpy(m_Name, other.Name(), sizeof(m_Name)); 
		strncpy(m_Value, other.Value(), sizeof(m_Value)); 
		strncpy(m_DefValue, other.m_DefValue, sizeof(m_DefValue));
		
		SetSetStatement(other.m_Set);
		SetGetStatement(other.m_Get);
		SetQuoted(other.IsQuoted());
		SetGenerated(other.IsGenerated());
	}
	
	virtual const TComponent *Owner() const { return m_Owner; }
	virtual const char *Name() const { return m_Name; }

	virtual TPropertyEditor *Editor() const { return m_Editor; }
	virtual void SetEditor(TPropertyEditor *editor) { m_Editor = editor; }
	
	void SetValue(const char *value) { if(value) strncpy(m_Value, value, sizeof(m_Value)); }
	const char *Value() const { return m_Value; }	
	
	void SetDefValue(const char *value) { if(value) strncpy(m_DefValue, value, sizeof(m_DefValue)); }
	const char *DefValue() const { return m_DefValue; }	
	
	bool IsDefValue(void) const { return(0 == strcmp(m_Value, m_DefValue)); }
	
	virtual void SetSetStatement(const char *set) { if(set) strncpy(m_Set, set, sizeof(m_Set)); }
	virtual const char *SetStatement(void) const;
	
	virtual void SetGetStatement(const char *get) { if(get) strncpy(m_Get, get, sizeof(m_Get)); }
	virtual const char *GetStatement(void) const;
	
	void SetQuoted(bool b)  { m_Quoted = b; }
	bool IsQuoted(void) const { return m_Quoted; }
	
	void SetGenerated(bool b) { m_Generated = b; }
	bool IsGenerated(void) const { return m_Generated; }
	
	// Called by the form designer to give the property a chance to write
	// xtended property info to the form file,
	virtual void WriteExtendedData(BMessage *) {}
	// or read it.
	virtual void ReadExtendedData(BMessage *) {}
	// Actual implementation of extended property data is left completely
	// up to property classes that derive from this one.
private:
	char m_Name[T_PROP_NAME_LENGTH];
	TComponent *m_Owner;
	TPropertyEditor *m_Editor;
	char m_Value[T_PROP_VALUE_LENGTH];
	char m_DefValue[T_PROP_VALUE_LENGTH];
	char m_Set[T_PROP_VALUE_LENGTH];
	char m_Get[T_PROP_VALUE_LENGTH];
	bool m_Quoted;
	bool m_Generated;
};

//
// An event.
// 
class TEvent {
public:
	TEvent(
		const char *name
	, const char *params = NULL
	, const char *returntype = NULL
	, bool selected = false
	);
	
	const char *Name() const 	  { return m_Name; 	 }
	const char *Params() const  { return m_Params; }
	const char *RetType() const { return m_RetType;}
	
	void SetSelected(bool sel)  { m_Selected = sel; }
	bool IsSelected(void) const { return m_Selected; }
private:
	const char *m_Name;
	const char *m_Params;
	const char *m_RetType;
	bool m_Selected;
};

//
// Represents a Component. VBe++ components derive off of real controls, plus this, which gives
// them the attributes they need to live in the design environment.
//
class TComponent {
public:
	TComponent(TComponentOwner *cOwner = NULL, TFormDesigner *Designer = NULL, TComponentType type = ctVisual)
	:	m_Owner(cOwner)
	, m_Designer(Designer)
	, m_Type(type)
	, m_Properties(new BList())
	, m_Events(new BList())
	, m_PropListeners(new BList())
	, m_Mode(cmRuntime)
	, m_Selected(false)
	, m_Store(new TPropertyStore(this))
	, m_Modified(false)
	, m_DTView(0)
	, m_TabOrder(-1)
	, m_IsChildView(true)
	{
	}
	
	virtual ~TComponent() 
	{
		ClearProperties();
		delete m_Properties;
		
		ClearEvents();
		delete m_Events;
		
		ClearPropertyListeners();
		delete m_PropListeners;
		
		delete m_Store;
	}
	
	void SetClassName(const char *name) { if(name) strncpy(m_ClassName, name, sizeof(m_ClassName)); }
	const char *ClassName(void) const { return m_ClassName; }

	virtual void SetComponentName(const char *name) { if(name) strncpy(m_Name, name, sizeof(m_Name)); }
	virtual const char *ComponentName(void) const { return m_Name; }
	
	virtual void SetComponentType(TComponentType type) { m_Type = type; }
	virtual TComponentType ComponentType(void) const { return m_Type; }
	
	virtual void SetOwner(TComponentOwner *Owner) { m_Owner = Owner; }
	virtual TComponentOwner *Owner() const { return m_Owner; }
	
	virtual void SetComponentMode(TComponentMode mode) { m_Mode = mode; }
	virtual TComponentMode ComponentMode(void) const { return m_Mode; }
	
	virtual void SetDesigner(TFormDesigner *Designer) { m_Designer = Designer; }
	virtual TFormDesigner *Designer() const { return m_Designer; }
	
	virtual void DesignerSelect(bool select) { m_Selected = select; }
	virtual bool IsDesignerSelected(void) const { return m_Selected; }
	
	virtual void SetModified(bool mod = true) { m_Modified = mod; }
	virtual bool Modified(void) const { return m_Modified; }
	
	virtual void SetTabOrder(int32 order) { m_TabOrder = order; }
	virtual int32 TabOrder(void) const { return m_TabOrder; }
	
	// called by StoreProperty() just before a property value is changed.
	// returning false from this function cancels the change.
	virtual bool PropertyModified(const char *name, const char *oldval, const char *newval) { return true; }
	
	virtual void DesignerDrawSelection(void);
	
	// called to let a component add items to its context menu
	virtual bool AddComponentContextMenuItems(BPopUpMenu *) { return false; }
	// called when an item on the context menu is selected
	virtual bool HandleComponentContextMessage(BMessage *) { return false; }
	// called when the control is double-clicked
	virtual bool DefaultEditAction(BPoint pt) { return false; }
	// called to process design-environment messages
	virtual bool HandleComponentDesignMessage(BMessage *);
	
	// Design-time Events
	virtual bool CanDropOnDesignerAt(BPoint location) { return true; }
	virtual void DroppedOnFormDesigner(BRect location);
	virtual void SelectedInFormDesigner(void);
	virtual void InvokedInFormDesigner(void);
	virtual void PopupMenuInvokedInFormDesigner(BPopUpMenu *menu);
	virtual void MenuItemSelectedInFormDesigner(BMenuItem *item);
	virtual void DeletedFromFormDesigner(void);
	virtual void AddedToPalette(void);
	virtual void PopupMenuInvokedInPalette(BPopUpMenu *menu);
	virtual void MenuItemSelectedInPalette(BMenuItem *item);
	virtual void DeletedFromPalette(void);
	virtual void DesignerMouseDown(BPoint pt, BRect bounds, BView *view);
	
	// Property-management functions
	int32 CountProperties(void) const { return m_Properties->CountItems(); }
	TProperty *PropertyAt(int32 index) { return (TProperty *)m_Properties->ItemAt(index); }
	TProperty *FindProperty(const char *name) 
	{ 
		for(int i=0; i<CountProperties();i++)
		{
			if(0 == strcmp(PropertyAt(i)->Name(), name))
				return PropertyAt(i);
		} 
		return 0;
	}
	
	// Design-time management
	virtual void MoveComponentTo(BView *vw, BPoint pt);
	virtual void MoveComponentTo(BPoint pt);
	virtual void ResizeComponentTo(BView *vw, BRect rect) { vw->ResizeTo(rect.Width(), rect.Height()); };
	virtual void ResizeComponentTo(BRect rect) { if(m_DTView) m_DTView->ResizeTo(rect.Width(), rect.Height());}
	
	void SetDTView(BView *vw) { m_DTView = vw; }
	BView *DTView(void) const { return m_DTView; }

	// Events/overrides management functions
	int32 CountEvents(void) const { return m_Events->CountItems(); }
	TEvent *EventAt(int32 index) { return (TEvent *)m_Events->ItemAt(index); }
	TEvent *FindEvent(const char *name) 
	{ 
		for(int i=0; i<CountEvents();i++)
		{
			if(0 == strcmp(EventAt(i)->Name(), name))
				return EventAt(i);
		} 
		return 0;
	}
	
	// Property listeners: Property editors that request to be notified whenever
	// a property in this component is changed.
	int32 CountPropertyListeners(void) const { return m_PropListeners->CountItems(); }
	TPropertyEditor *PropertyListenerAt(int32 index) { return (TPropertyEditor *)m_PropListeners->ItemAt(index); }
	
	//
	// These are implemented by a component
	// class that multiply derives from an actual control and this class.
	//
	// Retrieve the property value (as text, no matter what) for the property
	// referenced by "propName"
	virtual const char *GetPropertyValue(const char *propName) const;
	// Set the property value (converting from text if nec.) for the property
	// referenced by "propName"
	virtual void SetPropertyValue(const char *propName, const char *value);
	// Called to store the property. Do NOT call PropertyStore()->SetString(),
	// unless you don't want the new property value stored in the actual property.
	// You don't want this.
	void StoreProperty(const char *name, const char *value);
	
	// Return whether the specified event is selected (will be generated)
	virtual bool EventSelection(const char *name) const;
	// Set whether the specified event is selected 
	virtual void SetEventSelection(const char *name, bool selected);
	
	
	//
	// These functions should be called from derived classes' constructors.
	//
	// Load the list of properties for this component
	virtual void InitPropertyList(void);
	// Load the list of events for this component
	virtual void InitEventList(void);
	// Initialize design-time property values
	virtual void InitPropertyValues(void);
	// Initialize design-time event values
	virtual void InitEventValues(void);	
	
	// Property storage 
	TPropertyStore *PropertyStore(void) const { return m_Store; }
	
	// Used to write specialized data to the form file
	virtual void WriteExtendedComponentDataTo(BMessage *) {}
	virtual void ReadExtendedComponentDataFrom(BMessage *) {}
	
	// Returns the base name of the html file used for documenting the component;
	// Visual Be++ creates a command line based on this:
	// VBe++ Component Directory + component->HTMLFile() + "#" + component->ClassName()
	// and passes that to Net+ to display documentation for the component.
	// (That is, if this function returns something other than the default NULL.)
	virtual const char *HTMLFile(void) const { return 0; }
	
	//
	// Code Generation functions
	//
	// Returns the name of the header file for this component
	virtual const char *HeaderFile(void) const { return 0; }
	virtual const char *LibFile(void) const { return 0; }

	// Called by the generator to see if the component wants to generate any code
	// before it's constructed in the generated code.
	virtual bool OnBeforeConstruct(BString &s) { return false; }
	// Called by the generator in case the component wants to generate any code immediately
	// after it's constructed in the generated code.
	virtual bool OnAfterConstruct(BString &s) { return false; }
	// Called by the generator to ask if a constructor call is generated
	// by the component instead of created by the generator
	virtual bool OnGenConstructorCall(BString &s) { return false; }
	// Called when asking for constructor params; The rules for constructor
	// generation are:
	// If the IsChildView() function of this component returns true:
	// If this component is of type ctVisual, the code generator generates a 
	// constructor call that passes "this" (the form), a BRect of the component's position
	// on the form, and the component's name.
	// If this component is of type ctNonVisual, the code generator generates the same
	// constructor call, but omits the BRect parameter.
	// In both cases, the component is added via a call to AddChild().
	// If the IsChildView() function of this component returns false, the constructor
	// call does not call AddChild(), and passes "this", and this component's name.
	// The function below can add whatever parameters to the string it wants to after that point.
	virtual void OnSetConstructorParams(BString &s) {}
	// Called when generating the top of the header file
	virtual bool OnGenHeaderFileTop(BString &s) { return false; }
	// Called when generating the declaration section of the form's class
	virtual bool OnGenFormClassDecls(BString &) { return false; }
	// Called while generating "InitForm" 
	virtual bool OnGenInitForm(BString &s) { return false; }
	// Called when generating "DestroyForm"
	virtual bool OnGenDestroyForm(BString &s) { return false; }
	// Called when generating event handlers for controls. This is NOT where
	// normal event handlers are generated; Only event handlers that are 
	// component-specific are generated here.
	virtual bool OnGenEventHandlerProtos(BString &) { return false; }
	virtual bool OnGenEventHandlers(bool, BString &, BString &) { return false; }
	// This function controls whether the code generator tries to add the
	// component to the form with an AddChild() statement, or just construct the object.
	virtual bool IsChildView(void) const { return m_IsChildView; }
	virtual void SetIsChildView(bool b) { m_IsChildView = b; }
protected:
	//
	// Property management
	//
	virtual void AddProperty(TProperty *prop)
	{ m_Properties->AddItem(prop); }
	
	virtual void AddPropAndEditor(
		TProperty *prop
	, TPropertyEditor *editor
	,	const char *setStatement = 0
	, const char *getStatement = 0
	, bool quoted = false
	);
	
	virtual bool RemoveProperty(TProperty *prop)
	{ return m_Properties->RemoveItem(prop); }
	
	virtual TProperty *RemoveProperty(int32 index)
	{ return (TProperty *)m_Properties->RemoveItem(index); }
	
	virtual void ClearProperties(bool deleteItems = true)
	{
		if(deleteItems)
		{
			TProperty *prop;
			for(int i=0; (prop=(TProperty *)m_Properties->ItemAt(i)); i++)
				delete prop;
		}
		m_Properties->MakeEmpty();
	}
	
	//
	// Event list management
	//
	virtual void AddEvent(TEvent *evt)
	{ m_Events->AddItem(evt); }
	
	virtual bool RemoveEvent(TEvent *evt)
	{ return m_Events->RemoveItem(evt); }
	
	virtual TEvent *RemoveEvent(int32 index)
	{ return (TEvent *)m_Events->RemoveItem(index); }
	
	virtual void ClearEvents(bool deleteItems = true)
	{
		if(deleteItems)
		{
			TEvent *evt;
			for(int i=0; (evt=(TEvent *)m_Events->ItemAt(i)); i++)
				delete evt;
		}
		m_Events->MakeEmpty();
	}

public:	
	//
	// Property Listener management
	//
	virtual void AddPropertyListener(TPropertyEditor *ed)
	{ m_PropListeners->AddItem(ed); }
	
	virtual bool RemovePropertyListener(TPropertyEditor *ed)
	{ return m_PropListeners->RemoveItem(ed); }
	
	virtual TPropertyEditor *RemovePropertyListener(int32 index)
	{ return (TPropertyEditor *)m_PropListeners->RemoveItem(index); }
	
	virtual void ClearPropertyListeners(void)
	{
		m_PropListeners->MakeEmpty();
	}
	
	
protected:
	char m_Name[T_PROP_VALUE_LENGTH];
	char m_ClassName[T_COMPONENT_CLASSNAME_LENGTH];
	TComponentOwner *m_Owner;
	TFormDesigner *m_Designer;
	TComponentType m_Type;
	BList *m_Properties;
	BList *m_Events;
	BList *m_PropListeners;
	TComponentMode m_Mode;
	bool m_Selected;
	TPropertyStore *m_Store;
	bool m_Modified;

private:	
	BView *m_DTView; // design-time view
	int32 m_TabOrder;
	bool m_IsChildView;
};

//
// Represents an object that owns other components
//
class TComponentOwner {
public:
	TComponentOwner(void)
	: m_Components(new BList)
	{}
	
	virtual ~TComponentOwner(void);
	
	// Called by TComponent::StoreProperty() just before a property is changed.
	// Can return false to cancel the change. Note that this is called AFTER
	// the control has already been allowed to process the change.
	virtual bool ComponentPropertyModified(
		TComponent *comp
	, const char *propName
	, const char *oldval
	, const char *newval
	) { return true; }
	
	// Component list management
	virtual int32 CountComponents() const { return m_Components->CountItems(); }
	virtual TComponent *ComponentAt(int index) const { return (TComponent *)m_Components->ItemAt(index); }
	
	BList *ComponentList(void) const { return m_Components; }
	
	virtual void AddComponent(TComponent *comp)
	{ 
		m_Components->AddItem(comp); 
		comp->SetOwner(this); 
		comp->SetTabOrder(MaxTabOrder()+1); 
	}

	virtual bool RemoveComponent(TComponent *comp)
	{ 
		bool result = false;
		comp->SetOwner(0); 
		result = m_Components->RemoveItem(comp); 
		SetTabOrders();
		return result;
	}	

	virtual TComponent *RemoveComponent(int32 index)
	{ 
		TComponent *result = (TComponent *)m_Components->RemoveItem(index);
		if(result) result->SetOwner(0);
		SetTabOrders();
		return result;
	}	
	
	virtual void ClearComponents(bool deleteItems = false)
	{
		if(deleteItems)
		{
			TComponent *comp;
			for(int i=0; (comp=(TComponent *)m_Components->ItemAt(i)); i++)
				if(comp) 
					delete comp;
		}
		m_Components->MakeEmpty();
	}
	
	virtual int32 MaxTabOrder(void) 
	{
		int32 result = -1;
		for(int32 i = 0; i < CountComponents(); i++)
		{
			TComponent *comp = ComponentAt(i);
			if(comp->TabOrder() > result)	result = comp->TabOrder();
		}
		return result;	
	}
	
	virtual void SetTabOrders(void);

protected:
	BList *m_Components;
};


//
// Represents an application component
//
class TAppComponent: public TComponent, public TComponentOwner {
public:
	TAppComponent(void)
	: TComponent(NULL, NULL, ctNonVisual)
	{
		SetClassName("TMyApplication");
		InitPropertyList();
		InitEventList();
	}
	
	// Load the list of properties for this component
	virtual void InitPropertyList(void);
	// Load the list of events for this component
	virtual void InitEventList(void);
	// Initialize design-time property values
	virtual void InitPropertyValues(void);
	// Initialize design-time event values
	virtual void InitEventValues(void);	

	virtual const char *GetPropertyValue(const char *propName) const;
	// Set the property value (converting from text if nec.) for the property
	// referenced by "propName"
	virtual void SetPropertyValue(const char *propName, const char *value);
};

//
// Represents a non-visual component
//
class TNonVisualComponent: public TComponent {
public:
	TNonVisualComponent(TComponentOwner *cOwner = NULL, TFormDesigner *Designer = NULL)
	: TComponent(cOwner, Designer, ctNonVisual)
	{}
	
	virtual void InitPropertyList(void);
	virtual void InitEventList(void);
	virtual void InitPropertyValues(void);
	virtual void InitEventValues(void);
	virtual const char *GetPropertyValue(const char *name) const;
	virtual void SetPropertyValue(const char *name, const char *value);
	virtual void OnSetConstructorParams(BString &s);
};

//
// Represents a visual component
//
class TVisualComponent: public TComponent {
public:
	TVisualComponent(TComponentOwner *cOwner = NULL, TFormDesigner *Designer = NULL)
	: TComponent(cOwner, Designer, ctVisual)
	{}

	// Load the list of properties for this component
	virtual void InitPropertyList(void);
	// Load the list of events for this component
	virtual void InitEventList(void);
	// Initialize design-time property values
	virtual void InitPropertyValues(void);
	// Initialize design-time event values
	virtual void InitEventValues(void);	

	virtual const char *GetPropertyValue(const char *propName) const;
	// Set the property value (converting from text if nec.) for the property
	// referenced by "propName"
	virtual void SetPropertyValue(const char *propName, const char *value);
	
	virtual void OnSetConstructorParams(BString &s);
};

//
// Represents a BView-based component
//
class TViewComponent: public TVisualComponent {
public:
	TViewComponent(TComponentOwner *cOwner = NULL, TFormDesigner *Designer = NULL)
	: TVisualComponent(cOwner, Designer)
	{}

	// Load the list of properties for this component
	virtual void InitPropertyList(void);
	// Load the list of events for this component
	virtual void InitEventList(void);
	// Initialize design-time property values
	virtual void InitPropertyValues(void);
	// Initialize design-time event values
	virtual void InitEventValues(void);	

	virtual const char *GetPropertyValue(const char *propName) const;
	// Set the property value (converting from text if nec.) for the property
	// referenced by "propName"
	virtual void SetPropertyValue(const char *propName, const char *value);
	
	virtual void OnSetConstructorParams(BString &s);
};

//
// Represents a form component
//
class TFormComponent: public TVisualComponent {
public:
	TFormComponent()
	: TVisualComponent(NULL, NULL)
	{}

	// Load the list of properties for this component
	virtual void InitPropertyList(void);
	// Load the list of events for this component
	virtual void InitEventList(void);
	// Initialize design-time property values
	virtual void InitPropertyValues(void);
	// Initialize design-time event values
	virtual void InitEventValues(void);	

	virtual const char *GetPropertyValue(const char *propName) const;
	// Set the property value (converting from text if nec.) for the property
	// referenced by "propName"
	virtual void SetPropertyValue(const char *propName, const char *value);
};

// 
// Represents a control component
//
class TControlComponent: public TViewComponent {
public:
	TControlComponent(TComponentOwner *cOwner, TFormDesigner *Designer, TComponentType type)
	: TViewComponent(cOwner, Designer)
	{}
	
	// Load the list of properties for this component
	virtual void InitPropertyList(void);
	// Load the list of events for this component
	virtual void InitEventList(void);
	// Initialize design-time property values
	virtual void InitPropertyValues(void);
	// Initialize design-time event values
	virtual void InitEventValues(void);	

	virtual const char *GetPropertyValue(const char *propName) const;
	// Set the property value (converting from text if nec.) for the property
	// referenced by "propName"
	virtual void SetPropertyValue(const char *propName, const char *value);
};

//
// Form Designer. This class serves as one of the ancestors for a TForm within the VBe++ environment.
// It provides various features for communicating with controls.
//
class TFormDesigner: public TComponentOwner {
public:
	TFormDesigner()
	: TComponentOwner()
	, m_Locked(false)
	{}
	
	virtual ~TFormDesigner()
	{
		ClearComponents();
		delete m_Components;
	}
	
	// Design-time events
	virtual void DesignFormShown(bool shown) {}
	virtual void DesignFormActivated(bool activated) {}
	virtual void ComponentSelected(TComponent *component, bool selected = true) {}
	virtual void ComponentMoved(TComponent *component, BPoint whereTo) {}
	virtual void ComponentResized(TComponent *component, float width, float height) {}
	
	// used by parent controls (group-boxes, etc.) when creating new
	// child controls
	virtual bool IsDropWaiting(void) const { return false; }	
	virtual TComponent *CreateComponent(BView *view, BRect rect) { return 0; }
	virtual void DoneDroppingComponent(void) {}
	
	virtual BRect DragMsgBounds(BRect r) { return r; }
	
	virtual void UpdateComponentPosition(TComponent *, BRect *) {}
	
	// Used by parent controls (group boxes, etc.) when moving multiply-selected
	// controls around on their surfaces.
	virtual uint32 SelectionCount(void) const { return 0; }
	virtual TComponent *SelectedComponentAt(uint32) { return 0; }
	
	BView *DTFormView(void) const { return m_DTFormView; }
	
	void SetLocked(bool b) { m_Locked = b; }
	bool Locked(void) const { return m_Locked; }
	
protected:
	virtual void SetDTFormView(BView *vw) { m_DTFormView = vw; }
	
private:
	BView *m_DTFormView;
	TComponent *m_DropCompoment;
	bool m_Locked;
};

//
// App designer. This class serves to provide app-specific
// features in the design environment.
//
class TAppDesigner {
public:
	TAppDesigner()
	{}
	virtual void DesignAppShown(bool shown) {}
	virtual void DesignAppActivated(bool activated) {}	
};

//
// A base Property editor.
// 
class TPropertyEditor {
public:
	TPropertyEditor(TComponent *cComponent, TProperty *prop, TPropEditType type)
	:	m_Component(cComponent)
	,	m_Prop(prop)
	, m_Type(type)
	, m_Common(true)
	{
		ClearValidateErrorMessage();
	}
	
	virtual ~TPropertyEditor() {}
	
	void SetProperty(TProperty *prop) { m_Prop = prop; }
	TProperty *Property() const { return m_Prop; }
	TComponent *Component() const { return m_Component; }
	TPropEditType Type() const { return m_Type; }
	
	// Called when a property is selected in the Properties window. 
	virtual void Activate() {}
	// Called when the user presses the "..." button or double-clicks a property
	// in the Properties window. (petModalEdit type). The TComponent ** param
	// is an array of components. Each component in the array should have the
	// changes your modal editor makes to the property applied to it.
	// There should always be at least one component in the array.
	// The uint32 parameter specifies the number of components in the array.
	virtual bool ModalEdit(TComponent **, uint32) { return false; }
	// Called by SetValue() when the property has been modified.
	virtual void Modified() {}
	// Called after the editor is created, but before it is used.
	virtual void Initialize() {}
	// Called to get the edit limit for a string property.
	virtual int32 EditLimit() { return T_PROP_VALUE_LENGTH; }
	// Called to set the value of the property. "setControlToo" indicates
	// whether the component should be asked to set its value.
	virtual void SetValue(const char *value, bool setControlToo = true);
	
	// Called to retrieve the text value of the property.
	virtual const char *GetValue(void) const
	{
		return m_Prop->Value();
	} 
	
	// Called to get a string that can be displayed in a property editor.
	// By default it returns the value of the underlying property.
	virtual const char *GetDisplayValue(void) const	{ return GetValue(); }
	
	// Called to retrieve a list of values for a pick list.
	virtual void GetValuePickList(TStringList *list) {}
	// Called to retrieve a list of values for a regular list.
	virtual void GetValueList(TStringList *list) {}
	// Called by the property window to validate a typed-in value.
	virtual bool Validate(const char *) { return true; }
	
	// Called by the property editor to set the validation error message.
	virtual void SetValidateErrorMessage(const char *msg) { if(msg) strncpy(m_ValidateErrorMessage, msg, sizeof(m_ValidateErrorMessage)); }
	// Clears the validation error message
	virtual void ClearValidateErrorMessage(void) { memset(m_ValidateErrorMessage, 0, sizeof(m_ValidateErrorMessage)); }
	// Returns the currently-set validate error message
	virtual const char *ValidateErrorMessage(void) const { return m_ValidateErrorMessage; } 
	
	// Called by a component for every property editor who requests to be
	// notified when a property in a component changes; If 
	// a property editor adds itself to the list of listeners for a property
	// in a component, and the value of the property is changed, the component
	// will call this function in each property editor in its list of listeners.
	virtual void PropListenerNotify(TComponent *c, const char *propname, const char *oldvalue, const char *newvalue) {}
	
	// Set or determine whether a property can be displayed in
	// a multi-select situation.
	void SetCommon(bool b) { m_Common = b; }
	bool IsCommon(void) const { return m_Common; }
private:
	TComponent *m_Component;
	TProperty *m_Prop;
	TPropEditType m_Type;
	bool m_Common;
	char m_ValidateErrorMessage[256];
};

bool isnumeric(const char *);

//
// Basic modal-edit dialog class
//
enum {
	PME_BTNAPPLY = 0x1
, PME_BTNCANCEL
};

//
// A base class for modal-edit property windows.
//
class TPropertyModalEdit: public BWindow {
public:
	TPropertyModalEdit(TPropertyEditor *editor, char *value, uint32 maxlen, TComponent **selected, uint32 count)
	: BWindow(BRect(100,100,300,300), "Modal Edit", B_MODAL_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE)
	, m_MaxLen(maxlen)
	, m_Editor(editor)
	, m_Selected(selected)
	, m_Count(count)
	{
		if(value) strncpy(m_Value, value, sizeof(m_Value));
		
		InitWindow();
		InitControls();
		Show();
	}
	
	virtual ~TPropertyModalEdit(void)
	{
		if(m_Selected)
			delete [] m_Selected;
	}
	
	// Create the required controls on the window here
	virtual void InitWindow(void) {}
	// Initialize the controls based on the current property value here.
	virtual void InitControls(void) {}
	// Called when the OK/Apply/Save button (whatever you call it) is pressed.
	virtual void OnApply(void) = 0;
	// Called when the Cancel/Close button is pressed.
	virtual void OnCancel(void) { Quit(); }
	// Called when the window receives a message
	virtual bool OnMessage(BMessage *) { return false; }
	
	// BWindow overrides
	virtual void MessageReceived(BMessage *message)
	{
		switch(message->what)
		{
			case PME_BTNAPPLY:
			{
				OnApply();
				
				// Apply the changes to the property editor.
				if(m_Editor && m_Editor->Property())
				{
					// apply the change to each component in the m_Selected array.
					for(uint32 i = 0; i < m_Count; i++)
					{
						TComponent *c = m_Selected[i];
						if(c)
						{
							TProperty *prop = c->FindProperty(m_Editor->Property()->Name());
							if(prop && prop->Editor())
							{
								prop->Editor()->SetValue(m_Value);
								c->SetPropertyValue(m_Editor->Property()->Name(), m_Value);
							}
						}
					}
				}
				
				Quit();
				break;
			}
			
			case PME_BTNCANCEL:
			{
				OnCancel();
				Quit();
				break;
			}
			
			default:
			{
				if(!OnMessage(message))
				{
					BWindow::MessageReceived(message);
				}
			}
		}
	}
	
protected:
	uint32 m_MaxLen;
	char m_Value[T_PROP_VALUE_LENGTH];
	TPropertyEditor *m_Editor;
	TComponent **m_Selected;
	uint32 m_Count;
};

// basic integer property
class TIntProperty: public TPropertyEditor {
public:
	TIntProperty(TComponent *Component, TProperty *prop)
	: TPropertyEditor(Component, prop, petString)
	{}
	virtual int32 EditLimit() { return 63; }
	virtual bool Validate(const char*);
};

// basic string property
class TStringProperty: public TPropertyEditor {
public:
	TStringProperty(TComponent *Component, TProperty *prop)
	: TPropertyEditor(Component, prop, petString)
	{}
};

// component name property
class TNameProperty: public TStringProperty {
public:
	TNameProperty(TComponent *Component, TProperty *prop)
	: TStringProperty(Component, prop)
	{
		// This property is not displayed in a multi-select situation.
		SetCommon(false); 
	}
	
	virtual int32 EditLimit() { return 32; }
	virtual bool Validate(const char *);
};

// A Component-referencing property
class TComponentProperty: public TPropertyEditor {
public:
	TComponentProperty(TComponent *component, TProperty *prop)
	: TPropertyEditor(component, prop, petComponent)
	, m_ComponentRef(NULL)
	{
		SetCommon(false);
	}
	
	void SetComponentRef(TComponent *comp);
	TComponent *ComponentRef(void) const;
	void RemoveComponentRef(void);
	
	virtual const char *GetValue() const;
	virtual void GetValuePickList(TStringList *list);
	virtual const char *GetDisplayValue() const;
	virtual void PropListenerNotify(TComponent *c, const char *name, const char *oldval, const char *newval);
private:
	TComponent *m_ComponentRef;
};

// rgb color
rgb_color StrToRGBColor(const char *s, rgb_color *pclr);
const char *RGBColorToStr(rgb_color clr, char *buf);

class TRGBColorProperty: public TPropertyEditor {
public:
	TRGBColorProperty(TComponent *Component, TProperty *prop)
	:	TPropertyEditor(Component, prop, petModalEdit)
	{}
	
	virtual bool ModalEdit(TComponent **, uint32);
	virtual const char *GetDisplayValue() const { return "(rgb_color)"; }
};

class TViewColorProperty: public TRGBColorProperty {
public:
	TViewColorProperty(TComponent *Component, TProperty *prop)
	: TRGBColorProperty(Component, prop)
	{}
	virtual const char *GetDisplayValue() const { return "(rgb_color)"; }
};

// basic list property
class TStringListProperty: public TPropertyEditor {
public:
	TStringListProperty(TComponent *Component, TProperty *prop)
	: TPropertyEditor(Component, prop, petList)
	{}
	virtual void GetValueList(TStringList *list);
	virtual const char *GetDisplayValue() const { return "(List)"; }
	virtual void SetValueFromList(const TStringList *list);
	virtual bool ListItemIsSelected(const char *item) { return false; }
};

// basic pick list property
class TEnumProperty: public TPropertyEditor {
public:
	TEnumProperty(TComponent *Component, TProperty *prop)
	:	TPropertyEditor(Component, prop, petEnum)
	{}
	virtual void GetValuePickList(TStringList *list);
	virtual const char *GetDisplayValue() const;
};

// basic bool
class TBoolProperty: public TEnumProperty {
public:
	TBoolProperty(TComponent *Component, TProperty *prop)
	: TEnumProperty(Component, prop)
	{}
	
	virtual const char *GetDisplayValue() const { return GetValue(); }
	virtual void GetValuePickList(TStringList *list)
	{
		list->MakeEmpty();
		list->Add("true");
		list->Add("false");
	}
};

// Window look property
class TWindowLookProperty: public TEnumProperty {
public:
	TWindowLookProperty(TComponent *Component, TProperty *prop)
	: TEnumProperty(Component, prop)
	{}
	virtual void GetValuePickList(TStringList *list);
};

// Window Feel property
class TWindowFeelProperty: public TEnumProperty {
public:
	TWindowFeelProperty(TComponent *Component, TProperty *prop)
	: TEnumProperty(Component, prop)
	{}
	virtual void GetValuePickList(TStringList *list);
};

// Window Flags property
class TWindowFlagsProperty: public TPropertyEditor {
public:
	TWindowFlagsProperty(TComponent *Component, TProperty *prop)
	:	TPropertyEditor(Component, prop, petModalEdit)
	{}
	
	virtual bool ModalEdit(TComponent **, uint32);
	virtual const char *GetDisplayValue() const { return "(Flags)"; }
};

// Resizing mode property
class TResizingModeProperty: public TStringListProperty {
public:
	TResizingModeProperty(TComponent *component, TProperty *prop)
	: TStringListProperty(component, prop)
	{}
	
	virtual void GetValueList(TStringList *list);
	virtual void SetValueFromList(const TStringList *list);
	virtual bool ListItemIsSelected(const char *item);
};

// ViewFlags property
class TViewFlagsProperty: public TPropertyEditor {
public:
	TViewFlagsProperty(TComponent *component, TProperty *prop)
	: TPropertyEditor(component, prop, petModalEdit)
	{}
	
	virtual bool ModalEdit(TComponent **, uint32);
	virtual const char *GetDisplayValue() const { return "(Flags)"; }
};

// Font-related properties
class TFontProperty: public TEnumProperty {
public:
	TFontProperty(TComponent *component, TProperty *prop)
	: TEnumProperty(component, prop)
	{}
	
	virtual void GetValuePickList(TStringList *list);
};

class TFontSizeProperty: public TEnumProperty {
public:
	TFontSizeProperty(TComponent *component, TProperty *prop)
	: TEnumProperty(component, prop)
	{}
	
	virtual void GetValuePickList(TStringList *list);
};


#endif // _ComponentDesign_H